/* ****************************************************************************
  This file is part of KBabel

  Copyright (C) 2002 by Stanislav Visnovsky
                            <visnovsky@kde.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  In addition, as a special exception, the copyright holders give
  permission to link the code of this program with any edition of
  the TQt library by Trolltech AS, Norway (or with modified versions
  of TQt that use the same license as TQt), and distribute linked
  combinations including the two.  You must obey the GNU General
  Public License in all respects for all of the code used other than
  TQt. If you modify this file, you may extend this exception to
  your version of the file, but you are not obligated to do so.  If
  you do not wish to do so, delete this exception statement from
  your version.

**************************************************************************** */
#ifndef GETTEXTIMPORTPLUGIN_H
#define GETTEXTIMPORTPLUGIN_H

#include <catalogfileplugin.h>

#include <tqstringlist.h>

class KURL;
class TQFile;
class TQTextCodec;

/* ****************************************************************************
  The class for importing GNU gettext PO files. As an extra information,
  it stores the list of all obsolete entries.
**************************************************************************** */

class GettextImportPlugin: public KBabel::CatalogImportPlugin
{
public:
    GettextImportPlugin(TQObject* parent, const char* name, const TQStringList &);
    virtual KBabel::ConversionStatus load(const TQString& file, const TQString& mimetype);
    virtual const TQString id() { return "GNU gettext"; }

private:
    TQTextCodec* codecForArray(TQByteArray& arary, bool* hadCodec);
    KBabel::ConversionStatus readHeader(TQTextStream& stream);
    KBabel::ConversionStatus readEntry(TQTextStream& stream);
    
    // description of the last read entry
    TQString _msgctxt;
    TQStringList _msgid;
    TQStringList _msgstr;
    TQString _comment;
    bool _gettextPluralForm;
    bool _obsolete;
};

#endif
